@@ -1,21 +1,20 @@
 import os
 import sys
 from rchitect import rcopy, reval, rcall
-from rchitect._cffi import ffi, lib
-from rchitect.interface import roption, protected, rstring_p
+from rchitect.interface import roption
 from .key_bindings import map_key
-from .console import suppress_stderr
 
+def is_ascii(str):
+    return all(ord(c) < 128 for c in str)
 
-def prase_text_complete(text):
-    status = ffi.new("ParseStatus[1]")
-    s = rstring_p(text)
-    orig_stderr = sys.stderr
-    sys.stderr = None
-    with protected(s), suppress_stderr():
-        lib.R_ParseVector(s, -1, status, lib.R_NilValue)
-        sys.stderr = orig_stderr
-    return status[0] != 2
+def is_long_non_ascii_multiline(text):
+    if is_ascii(text):
+        return False
+    if "\n" not in text:
+        return False
+    if len(text) < 1000:
+        return False
+    return True
 
 
 def package_is_loaded(pkg):
@@ -81,6 +80,14 @@ def load_custom_key_bindings(*args):
     for m in esc_keymap:
         map_key(("escape", m["key"]), m["value"], mode=m["mode"] if "mode" in m else "r")
 
+    keymap = roption("radian.ctrl_key_map", [])
+    for m in keymap:
+        if m["key"] in "mihdc":
+            print("WARNING: Cannot remap c-" + m["key"] + ". Please remove this mapping from radian.ctrl_key_map in your radian profile")
+        else:
+            map_key(("c-" + m["key"],), m["value"], mode=m["mode"] if "mode" in m else "r")
+
+
 
 def register_cleanup(cleanup):
     rcall(("base", "reg.finalizer"),
@@ -89,12 +96,15 @@ def register_cleanup(cleanup):
           onexit=True)
 
 
-def set_lang():
+def set_locale(lc_ctype):
     if sys.platform.startswith("win"):
-        if not os.environ.get("LANG", ""):
-            if rcopy(reval(
-                   'compareVersion(paste0(R.version$major, ".", R.version$minor), "4.2.0") >= 0')):
-                os.environ["LANG"] = "en_US.UTF-8"
+        ucrt = rcopy(
+            reval('compareVersion(paste0(R.version$major, ".", R.version$minor), "4.2.0") >= 0'))
+        if ucrt:
+            if not os.environ.get("LANG", ""):
+                    os.environ["LANG"] = "en_US.UTF-8"
+            if lc_ctype:
+                reval("suppressWarnings(Sys.setlocale(locale = '{}'))".format(lc_ctype))
 
 
 def run_on_load_hooks():
